<?php

namespace itguild\forms\app\controllers;

use Illuminate\Database\Events\DatabaseBusy;
use itguild\forms\ActiveForm;
use itguild\forms\app\core\BaseController;
use itguild\forms\app\DTO\FormDTO;
use itguild\forms\app\models\FormInputModel;
use itguild\forms\app\models\FormModel;
use itguild\forms\app\models\InputTypeModel;
use itguild\forms\app\models\InputValueModel;
use itguild\forms\app\requests\CreateFormRequest;
use itguild\forms\core\cg_view\CgView;

/**
 * @var \itguild\forms\ActiveForm $form ;
 */

use itguild\forms\core\CgRequest;
use itguild\forms\debug\Debug;
use itguild\forms\exceptions\CreateFormException;
use itguild\forms\exceptions\FieldTypeNotFound;
use itguild\forms\exceptions\FormNotFoundException;
use itguild\forms\Form;
use itguild\forms\helpers\TranslitorHelper;
use itguild\forms\inputs\Select;
use itguild\forms\inputs\TextInput;
use itguild\forms\inputs\TextArea;
use itguild\forms\JsonForm;
use Twig\TwigFunction;

class AdminController extends BaseController
{

    public string $html;

    public function createAction(): void
    {
        $this->cgView->layout = "/layouts/admin_layout.php";
        $this->cgView->render("/admin/index.php", [
            'title' => "Редактировать форму",
            'form' => new ActiveForm(),
            'dto' => new FormDTO(),
        ]);
    }

    /**
     * @throws CreateFormException
     */
    public function saveFormAction(): void
    {
        $request = new CreateFormRequest();
        if (!$request->validate()) {

            throw new CreateFormException("Validation error");

        }

        $title = $request->post('title');
        $perPage = $request->post('perPage', 10);
        $inputs = $request->post('InputForm', []);

        $actionView = $request->post("actionView");
        $actionEdit = $request->post("actionEdit");
        $actionDelete = $request->post("actionDelete");
        $actions = [$actionView, $actionEdit, $actionDelete];


        $resultArray = ['meta' => ['perpage' => $perPage, 'status' => "1", 'actions' => $actions], 'params' => ['class' => "form-control", 'id' => 'form']];

        $form = FormModel::Create(['title' => $title, 'status' => FormModel::STATUS_ACTIVE, 'params' => json_encode($resultArray, true)]);

        foreach ($inputs as $input) {
            $name = TranslitorHelper::translit($input['name']);
            $params = [
                "name" => $name,
                "class" => "form-control",
            ];

            if ($input['type'] == InputTypeModel::RADIO_TYPE) {
                $radio = explode("\n", $input['radio']);

                foreach ($radio as $key) {
                    $nameRadio = TranslitorHelper::translit($key);
                    $paramsRadio = [
                        "name" => $name,
                        "value" => $key,
                        'label' => ['title' => $key, 'for' => $key, "class" => 'form-label']
                    ];
                    $formInput = FormInputModel::Create([
                        'form_id' => $form->id,
                        'input_type_id' => $input['type'],
                        'label' => $key,
                        'name' => $name,
                        'params' => json_encode($paramsRadio)
                    ]);
                }


            } else {

                $formInput = FormInputModel::Create([
                    'form_id' => $form->id,
                    'input_type_id' => $input['type'],
                    'label' => $name,
                    'name' => $name,
                    'params' => json_encode($params)
                ]);
            }

            if ($input['type'] == InputTypeModel::SELECT_TYPE) {
                $options = explode("\n", $input['options']);
                foreach ($options as $option) {
                    InputValueModel::Create(['form_input_id' => $formInput->id, 'value' => $option]);
                }
            }

        }

        $this->redirect("/form-result/$form->id");
    }


    public function getFormAddFieldsAction(): void
    {
        $this->cgView->render("/admin/_form_add_fields_ajax.php", ['form' => new ActiveForm()]);
    }

    public function getSelectValue(): void
    {
        $selectValue = $_GET["value"];
        $countInput = $_GET["count"];
        $form = new ActiveForm();

        $fields = [
            1 => '_checkBox',
            2 => '_text_input',
            3 => '_textarea',
            4 => '_radioButton',
            5 => '_button',
            6 => '_select'
        ];

        $viewName = $fields[$selectValue] ?? false;

        if (!$viewName) {
            throw new  FieldTypeNotFound($selectValue);
        }

        $this->cgView->render("/admin/field_type/$viewName.php", ['form' => new ActiveForm(), 'count' => $countInput]);
    }

    public function editFormAction($id)
    {
        $form = FormModel::find($id)->toArray();
        $dto = new FormDTO();

        $formArr = json_decode($form['params'], true);
        $fillableRes = [];
        $fillableRes['title'] = $form['title'];
        foreach ($formArr['meta'] as $key => $value){
            if($key == 'perpage'){
                $fillableRes['perPage'] = (string)$value;
            }
            else if($key == "actions"){
                foreach ($value as $item){
                    $fillableRes[$item] = $item;
                }
            }
        }
        $dto->load($fillableRes);

        if (!$form) {
            throw new FormNotFoundException($id);
        }

        $this->cgView->layout = "/layouts/admin_layout.php";
        $this->cgView->render("/admin/index.php", [
            'title' => "Редактировать форму",
            'form' => new ActiveForm(),
            'dto' => $dto,
        ]);
    }



}